#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <unistd.h>
#include <netdb.h>
#include "giFT.h"
#include "giFTpacket.h"
#include "main.h"

/**
 * The host, port, and pathname of the CGI script to use to upload the
 * nodes file.
 */
#define UPLOAD_HOST "gift.sourceforge.net"
#define UPLOAD_PORT 80
#define UPLOAD_PATH "/cgi-bin/nodes.cgi"

/**
 * The host, port, and pathname of the CGI script to use to download the
 * nodes file.
 */
#define DOWNLOAD_HOST "gift.sourceforge.net"
#define DOWNLOAD_PORT 80
#define DOWNLOAD_PATH "/cgi-bin/nodes.cgi"

/**
 * Make a half-hearted attempt to post the nodes file to the web
 * site, then exit.
 */
void nodesDoPost(const char *nodesFile)
{
    struct stat st;
    int res;
    byte *buf, *buf2, *d;
    size_t len, len2;
    int sock;
    int i;
    FILE *nodes, *sockf;
    struct sockaddr_in sin;
    struct hostent *hent;
    
    /* Read the nodes file. */
    nodes = fopen(nodesFile, "r");
    if (!nodes) goto done;

    res = fstat(fileno(nodes), &st);
    if (res < 0) goto done;

    buf = malloc(st.st_size+1);
    if (!buf) goto done;

    len = fread(buf, 1, st.st_size, nodes);
    if (len <= 0) goto done;

    fclose(nodes);

    /* We need to URL-escape the data */
    buf2 = malloc(3*len+1);
    if (!buf2) goto done;

    d = buf2;
    for(i=0;i<len;++i) {
	byte c = buf[i];
	if (strchr("%+/\\[]()<>'\" ", c) || c < ' ' || c > '~') {
	    sprintf(d, "%%%02x", c);
	    d += 3;
	} else {
	    *d = c;
	    d += 1;
	}
    }
    *d = '\0';
    len2 = d - buf2;

    /* Now try to send it to the web site. */
    sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (sock < 0) goto done;

    hent = gethostbyname(UPLOAD_HOST);
    if (!hent) goto done;

    sin.sin_family = AF_INET;
    sin.sin_port = htons(UPLOAD_PORT);
    memmove(&sin.sin_addr.s_addr,hent->h_addr,4);

    res = connect(sock, (struct sockaddr *)&sin, sizeof(sin));
    if (res < 0) goto done;

    sockf = fdopen(sock, "w");
    if (sockf) {
	fprintf(sockf, "POST " UPLOAD_PATH " HTTP/1.1\r\nHost: " UPLOAD_HOST
		"\r\nConnection: close"
		"\r\nContent-length: %d\r\n\r\n", len2 + 6 );
	fprintf(sockf, "nodes=");
	fwrite(buf2, len2, 1, sockf);
	fflush(sockf);
	fclose(sockf);
    } else {
	close(sock);
    }

done:
    exit(0);
}

/**
 * Fork a grandchild and have it call nodesDoPost().
 */
void nodesPost(const char *nodesFile)
{
    pid_t childpid, gchildpid;

    childpid = fork();
    if (childpid < 0) return;
    if (childpid > 0) {
	/* We're the parent.  Wait for the child, then return. */
	wait(NULL);
	return;
    }

    /* We're the child.  Fork a grandchild. */
    gchildpid = fork();
    /* The child should just exit. */
    if (gchildpid != 0) exit(0);

    /* Now we're the grandchild.  Do the dirty work. */
    nodesDoPost(nodesFile);
    /* Just in case... */
    exit(1);
}

/**
 * If we don't have a better source of node information, try to download
 * a nodes file from a web site.
 */
void nodesBlockingGet(const char *nodesFile)
{
    int sock = -1;
    int res;
    struct sockaddr_in sin;
    struct hostent *hent;
    FILE *sockf = NULL, *nodesf = NULL;
    char buf[1024];
    char *resg;
    char *space;

    /* Connect to the web site */
    sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
    if (sock < 0) goto done;

    hent = gethostbyname(DOWNLOAD_HOST);
    if (!hent) goto done;

    sin.sin_family = AF_INET;
    sin.sin_port = htons(DOWNLOAD_PORT);
    memmove(&sin.sin_addr.s_addr,hent->h_addr,4);

    res = connect(sock, (struct sockaddr *)&sin, sizeof(sin));
    if (res < 0) goto done;

    sockf = fdopen(sock, "r+");
    if (!sockf) goto done;
    sock = -1;

    /* Now copy the data into the nodes file. */
    nodesf = fopen(nodesFile, "w");
    if (!nodesf) goto done;

    /* Request the file.  We need to use HTTP/1.0, or else the server
     * might use some strange Tranfer-Encoding. */
    fprintf(sockf, "GET " DOWNLOAD_PATH " HTTP/1.0\r\nHost: " DOWNLOAD_HOST
	    "\r\nConnection: close\r\n\r\n");
    fflush(sockf);

    /* Read the result code */
    if (!fgets(buf, sizeof(buf), sockf)) goto done;
    space = strchr(buf, ' ');
    if (!space) goto done;
    if (strncmp(space, " 2", 2)) {
	/* Unsuccessful */
	goto done;
    }

    /* Read past the header */
    while (1) {
	resg = fgets(buf, sizeof(buf), sockf);
	if (resg == NULL) goto done;
	if (buf[0] == '\r' || buf[0] == '\n') break;
    }

    while(1) {
	/*
	res = fread(buf, 1, sizeof(buf), sockf);
	if (res <= 0) goto done;
	fwrite(buf, 1, res, nodesf);
	*/
	resg = fgets(buf, sizeof(buf), sockf);
	if (resg == NULL) break;
	fputs(buf, nodesf);
    }

    /* We're done. */
done:
    if (sock >= 0) close(sock);
    if (sockf) fclose(sockf);
    if (nodesf) fclose(nodesf);
}

/**
 * Read a potential list of supernodes from the nodes file.
 * Return the number of connections attempted.
 */
int nodesRead(const char *nodesFile, Connection *cs, size_t nconn,
	size_t maxactive, char *username, char *networkname,
	struct sockaddr_in *server_addr, time_t last_time)
{
    FILE *nodefp;
    struct stat st;
    int ret = 0;

    /* If the last modification time on the file is not later than
     * the last time we wrote to it, don't do anything. */
    if (!stat(nodesFile, &st) && st.st_mtime <= last_time) return 0;
    
    nodefp = fopen(nodesFile, "r");
    if (nodefp) {
	char buf[1024];
	while(fgets(buf, 1024, nodefp)) {
	    unsigned int a1,a2,a3,a4,p;
	    int res = sscanf(buf, "%u.%u.%u.%u:%u", &a1, &a2, &a3, &a4, &p);
	    if (res == 5) {
		struct sockaddr_in sin;
		socklen_t sinlen = sizeof(sin);
		sin.sin_family = AF_INET;
		sin.sin_port = htons(p);
		sin.sin_addr.s_addr = htonl(
			(a1<<24) | (a2<<16) | (a3<<8) | a4);
		addToConnectionList(cs, nconn, maxactive,
			(struct sockaddr *)&sin, sinlen,
			username, networkname, server_addr);
		++ret;
	    }
	}
	fclose(nodefp);
    }
    return ret;
}

/**
 * This AddressHandler is called once for each Connection in the
 * CONNECTED_SUPERNODE_CONNECTED state.  It writes the IP address and
 * the port number of the Connection to the FILE* passed in as the
 * context.
 */
static int addrwriter(int which, struct sockaddr *addr, socklen_t addrlen,
	void *ah_context)
{
    FILE *fp = ah_context;
    if (addr && addr->sa_family == AF_INET) {
	struct sockaddr_in *sin = (struct sockaddr_in *)addr;
	unsigned long a = ntohl(sin->sin_addr.s_addr);
	unsigned short p = ntohs(sin->sin_port);
	fprintf(fp, "%lu.%lu.%lu.%lu:%u\n", (a>>24)&0xff,
		(a>>16)&0xff, (a>>8)&0xff, a&0xff, p);
    }
    return 0;
}

/**
 * Write the current list of Connections in the ConnectionState
 * CONNECTED_SUPERNODE_CONNECTED to the nodes file.  Return
 * the last modification time to the file.
 */
time_t nodesWrite(const char *nodesFile, Connection *cs, size_t nconn)
{
    int res;
    struct stat st;
	FILE* nodefp;
	
    nodefp = fopen(nodesFile, "w");
    if (nodefp) {
	getConnecteds(cs, nconn, addrwriter, nodefp);
	fclose(nodefp);
    }

    /* Get the last modification time for the file. */
    res = stat(nodesFile, &st);
    if (res < 0) return 0;
    return st.st_mtime;
}
