# This is a shellscript "include" file that is included by most scripts here.

# Base tree of installation
if [ -z "$BASETREE" ] ; then
  # Assume we're included from a script in a subdir
  BASETREE="`dirname $0`/.."
fi

if [ ! -e "$BASETREE"/config.sh ] ; then
  echo "Problem: BASETREE isn't set correctly; can't find myself!"
  exit
fi


if [ "$HOSTNAME" = "myfancyfirewallbox" ] ; then
  # If your first network device is not your external connection, gnut will
  # advertise your internal IP address to the world; to correct this, enter
  # your "outside" IP address here.
  #
  #GNUTIP="12.34.56.78"

  # If you have assigned multiple IPs to your "outside" network card, you
  # can have the scanning appear to come from any one of them. (This may be
  # different from the GNUTIP.)
  #
  #NCIP="123.45.67.89"

  # And it still doesn't grok an empty if-then...
  :
fi


# The dir to store downloaded files (both during & after download)
#
DOWNLOADDIR="$HOME/shadowFT/downloads"


# Some dirs are needed for various classes of temporary files.
# Size: BIG/SMALL, limit about 10 MB; BIG should be on /home, not /tmp.
# Period: SHORT/LONG, limit about a day; LONG shouldn't be cleaned by
# some automagic /tmp cleaner.
# Availability: PUBLIC/PRIVATE; PUBLIC: everyone on an NFS network
# _needs_ to know where it is, so default under BASETREE (which is
# supposed to be on /home); if a specific file should be public only
# on one computer and not the entire NFS network, add a $HOSTNAME
# to it.
#
TEMPBIGSHORTPUBLIC="$BASETREE/pub"
TEMPBIGSHORTPRIVATE="$HOME/shadowFT/tmp"
TEMPBIGLONGPUBLIC="$BASETREE/pub"
TEMPBIGLONGPRIVATE="$HOME/shadowFT/tmp"
TEMPSMALLSHORTPUBLIC="$BASETREE/pub"
TEMPSMALLSHORTPRIVATE="/tmp/shadowFT-$USER/tmp"
TEMPSMALLLONGPUBLIC="$BASETREE/pub"
TEMPSMALLLONGPRIVATE="$HOME/shadowFT/tmp"


# The dir for the scanned indexes. Gets big, 20MB more every day.
# Here goes the invaluable stuff that we really can't lose.
#
SAVETREE="$BASETREE/indexes/$HOSTNAME/data"

# The host lists shouldn't be deleted ever; here also go the old indexes
# that should be kept a long time.
#
HOSTLISTSDIR="$BASETREE/indexes/$HOSTNAME"


# If set to 1, run a viewer program when a download has finished, unless
# "noview" is specified on the shadowFTget commandline.
# If set to 0, only run a viewer when "view" is specified.
# See the "viewers" file for details.
#
RUNVIEWER=0



##### Scan section

# Executables
NC="nc"
#NC="$HOME/src/netcat/nc"

MAKEBIGINDEX="$BASETREE/1-scan/makebigindex"
AUTOSCAN="$BASETREE/1-scan/shadowFTautoscan"
MANUSCAN="$BASETREE/1-scan/shadowFTmanuscan"
UPDATESCAN="$BASETREE/1-scan/shadowFTupdatescan"
STATS="$BASETREE/1-scan/shadowFTstats"
MULTICAT="$BASETREE/1-scan/multicat"

# Host lists
HOSTSTRIED="$HOSTLISTSDIR/hosts.tried"
HOSTSFAILED="$HOSTLISTSDIR/hosts.failed"
HOSTSEMPTY="$HOSTLISTSDIR/hosts.empty"
HOSTSFULL="$HOSTLISTSDIR/hosts.full"
HOSTSSUPER="$HOSTLISTSDIR/hosts.super"

# Where to store indexes that have been superceded by radically
# changed ones
OLDINDEXESBACKUP="$HOSTLISTSDIR/oldindexesbackup"

# "Manual scanning" default hosts-to-scan and hosts-scanned lists
# (overridable from commandline)
MANUSCANTOSCAN="$TEMPSMALLLONGPUBLIC/manuscan.hosts"
MANUSCANSCANNED="$TEMPSMALLLONGPUBLIC/manuscan.scanned"



##### Share section

# Executables
GNUTSFT="$BASETREE/2-share/gnut-sFT"
SHARESTART="$BASETREE/2-share/start-gnutshare"
SHARESTOP="$BASETREE/2-share/stop-gnutshare"

# Script restarting gnut when it stops (every half hour)
GNUTRUNNER="$BASETREE/2-share/gnutrunner"

# Log file
GNUTLOG="$TEMPSMALLLONGPRIVATE/gnutsFT.$HOSTNAME.log"

# Pid files (gnut restarted every time, runner runs always)
GNUTPIDFILE="$TEMPSMALLSHORTPRIVATE/gnutsFT.$HOSTNAME.pid"
GNUTPIDFILERUNNER="$TEMPSMALLLONGPRIVATE/gnutsFTrunner.$HOSTNAME.pid"

# Gnut script (only written at startup; only one gnut at a time (via pidfile))
GNUTRC="$TEMPSMALLLONGPRIVATE/gnutsFT.$HOSTNAME.rc"

# Time limit for checksum scan ("find" option; 4 days)
CHKFINDOPT="-mtime -4"
# Time limit for names scan ("find" option; 5 hours)
NAMFINDOPT="-mmin -300"



##### Search section

# Executables
GNUTSEARCH="$BASETREE/3-search/gnut-sFT-search"
RUNSEARCH="$BASETREE/3-search/run-gnutsearch"
KEEPSEARCHING="$BASETREE/3-search/keepsearching"

# Gnut scripts (used only during startup, not while running; multiple may
# run simultaneously)
GNUTRCSEARCH="$TEMPSMALLSHORTPRIVATE/gnutsearch.rc.$HOSTNAME.$$"



##### Get section

# Executables
GNUTWEB="$BASETREE/4-get/gnut-sFT-web"
GIFTGET="$BASETREE/4-get/giFTget"
UNMANGLE="$BASETREE/4-get/unmangle"
WEBSTART="$BASETREE/4-get/start-gnutweb"
WEBSTOP="$BASETREE/4-get/stop-gnutweb"
SFTGET="$BASETREE/4-get/shadowFTget"
HELPERGET="$BASETREE/4-get/helperget"

# Script restarting gnut when it stops (every hour)
GNUTRUNNERWEB="$BASETREE/4-get/gnutrunnerweb"

# Log file
GNUTLOGWEB="$TEMPSMALLLONGPRIVATE/gnutweb.log"

# Pid files (gnut restarted every time, runner runs always; runnerpid is
# used to check if thing runs)
GNUTPIDFILEWEB="$TEMPSMALLSHORTPRIVATE/gnutweb.pid"
GNUTPIDFILEWEBRUNNER="$TEMPSMALLLONGPUBLIC/gnutwebrunner.pid"

# Gnut script (only written at startup; only one gnut at a time (via pidfile))
GNUTRCWEB="$TEMPSMALLLONGPRIVATE/gnutweb.rc"

# Saved gnut-web config for IP and port (rewritten every hour)
GNUTWEBSAVEDCONFIG="$TEMPSMALLSHORTPUBLIC/gnutweb.savedconfig"

# Viewers file
VIEWERSFILE="$BASETREE/viewers"



##### Sharefiles section

# Executables
GIFT="$BASETREE/5-sharefiles/giFT"
SHAREFILESSTART="$BASETREE/5-sharefiles/start-gift"
SHAREFILESSTOP="$BASETREE/5-sharefiles/stop-gift"

# Pid file (giFT runs "forever", only one per host allowed (needs port 1214))
GIFTPIDFILE="$TEMPSMALLLONGPRIVATE/gift.$HOSTNAME.pid"



##### KZA section

# Executables, libraries
SCREEN="screen"
#SCREEN="$HOME/src/screen/screen"

KZA="$BASETREE/X-kza/kza"
#KZA="$HOME/kza/kza-0.4/kza"

KZAINPUT="$BASETREE/X-kza/kzainput"
LIBKZADUMP="$BASETREE/X-kza/libkzadump.so.1"
STDINSEMA="$BASETREE/X-kza/stdinsema"
KZAIPEXTRACT="$BASETREE/X-kza/kzaipextract"
KZASTART="$BASETREE/X-kza/start-kza"



# If someone else uses this and would like his/her own defaults, he/she can
# override them from a personal definitions file.
if [ -n "$SHADOWFTCONFIG" ] ; then
  . "$SHADOWFTCONFIG"
else
  if [ -r ~/.shadowFT.config ] ; then
    . ~/.shadowFT.config
  fi
fi


# Set up the necessary directories
mkdir -p "$TEMPBIGSHORTPUBLIC"
mkdir -p "$TEMPBIGSHORTPRIVATE"
mkdir -p "$TEMPBIGLONGPUBLIC"
mkdir -p "$TEMPBIGLONGPRIVATE"
mkdir -p "$TEMPSMALLSHORTPUBLIC"
mkdir -p "$TEMPSMALLSHORTPRIVATE"
mkdir -p "$TEMPSMALLLONGPUBLIC"
mkdir -p "$TEMPSMALLLONGPRIVATE"

mkdir -p "$SAVETREE"
mkdir -p "$HOSTLISTSDIR"
mkdir -p "$DOWNLOADDIR"


# Remember we already loaded the config
CONFIGLOADED=yes
